@@ -13,7 +13,7 @@ import torchode as to
 from torchdiffeq import odeint
 
 from beartype import beartype
-from beartype.typing import Tuple, Optional, List
+from beartype.typing import Tuple, Optional, List, Union
 
 from einops.layers.torch import Rearrange
 from einops import rearrange, repeat, reduce, pack, unpack
@@ -167,7 +167,7 @@ class LearnedSinusoidalPosEmb(Module):
 # https://arxiv.org/abs/2104.09864
 
 class RotaryEmbedding(Module):
-    def __init__(self, dim, theta = 10000):
+    def __init__(self, dim, theta = 50000):
         super().__init__()
         inv_freq = 1.0 / (theta ** (torch.arange(0, dim, 2).float() / dim))
         self.register_buffer("inv_freq", inv_freq)
@@ -176,8 +176,12 @@ class RotaryEmbedding(Module):
     def device(self):
         return self.inv_freq.device
 
-    def forward(self, seq_len):
-        t = torch.arange(seq_len, device = self.device).type_as(self.inv_freq)
+    @beartype
+    def forward(self, t: Union[int, Tensor]):
+        if not torch.is_tensor(t):
+            t = torch.arange(t, device = self.device)
+
+        t = t.type_as(self.inv_freq)
         freqs = torch.einsum('i , j -> i j', t, self.inv_freq)
         freqs = torch.cat((freqs, freqs), dim = -1)
         return freqs
@@ -258,21 +262,40 @@ class AdaptiveRMSNorm(Module):
 
 # attention
 
+class MultiheadRMSNorm(Module):
+    def __init__(self, dim, heads):
+        super().__init__()
+        self.scale = dim ** 0.5
+        self.gamma = nn.Parameter(torch.ones(heads, 1, dim))
+
+    def forward(self, x):
+        return F.normalize(x, dim = -1) * self.gamma * self.scale
+
 class Attention(Module):
     def __init__(
         self,
         dim,
         dim_head = 64,
         heads = 8,
-        dropout=0,
-        flash = False
+        dropout = 0,
+        flash = False,
+        qk_norm = False,
+        qk_norm_scale = 10
     ):
         super().__init__()
         self.heads = heads
-        self.scale = dim_head ** -0.5
         dim_inner = dim_head * heads
 
-        self.attend = Attend(dropout, flash = flash)
+        scale = qk_norm_scale if qk_norm else None
+
+        self.attend = Attend(dropout, flash = flash, scale = scale)
+
+        self.qk_norm = qk_norm
+
+        if qk_norm:
+            self.q_norm = MultiheadRMSNorm(dim_head, heads = heads)
+            self.k_norm = MultiheadRMSNorm(dim_head, heads = heads)
+
         self.to_qkv = nn.Linear(dim, dim_inner * 3, bias = False)
         self.to_out = nn.Linear(dim_inner, dim, bias = False)
 
@@ -282,6 +305,10 @@ class Attention(Module):
         q, k, v = self.to_qkv(x).chunk(3, dim = -1)
         q, k, v = map(lambda t: rearrange(t, 'b n (h d) -> b h n d', h = h), (q, k, v))
 
+        if self.qk_norm:
+            q = self.q_norm(q)
+            k = self.k_norm(k)
+
         if exists(rotary_emb):
             q, k = map(lambda t: apply_rotary_pos_emb(rotary_emb, t), (q, k))
 
@@ -310,12 +337,14 @@ class Transformer(Module):
         dim_head = 64,
         heads = 8,
         ff_mult = 4,
-        attn_dropout=0,
+        attn_dropout = 0,
+        num_register_tokens = 0.,
         attn_flash = False,
         adaptive_rmsnorm = False,
         adaptive_rmsnorm_cond_dim_in = None,
         use_unet_skip_connection = False,
-        skip_connect_scale = None
+        skip_connect_scale = None,
+        attn_qk_norm = False
     ):
         super().__init__()
         assert divisible_by(depth, 2)
@@ -323,6 +352,12 @@ class Transformer(Module):
 
         self.rotary_emb = RotaryEmbedding(dim = dim_head)
 
+        self.num_register_tokens = num_register_tokens
+        self.has_register_tokens = num_register_tokens > 0
+
+        if self.has_register_tokens:
+            self.register_tokens = nn.Parameter(torch.randn(num_register_tokens, dim))
+
         if adaptive_rmsnorm:
             rmsnorm_klass = partial(AdaptiveRMSNorm, cond_dim = adaptive_rmsnorm_cond_dim_in)
         else:
@@ -337,27 +372,58 @@ class Transformer(Module):
             self.layers.append(nn.ModuleList([
                 nn.Linear(dim * 2, dim) if has_skip else None,
                 rmsnorm_klass(dim = dim),
-                Attention(dim=dim, dim_head=dim_head, heads=heads, dropout=attn_dropout, flash=attn_flash),
+                Attention(dim = dim, dim_head = dim_head, heads = heads, dropout = attn_dropout, flash = attn_flash, qk_norm = attn_qk_norm),
                 rmsnorm_klass(dim = dim),
                 FeedForward(dim = dim, mult = ff_mult)
             ]))
 
         self.final_norm = RMSNorm(dim)
 
+    @property
+    def device(self):
+        return next(self.parameters()).device
+
     def forward(
         self,
         x,
         mask = None,
         adaptive_rmsnorm_cond = None
     ):
+        batch, seq_len, *_ = x.shape
+
+        # add register tokens to the left
+
+        if self.has_register_tokens:
+            register_tokens = repeat(self.register_tokens, 'n d -> b n d', b = batch)
+
+            x, ps = pack([register_tokens, x], 'b * d')
+
+            if exists(mask):
+                mask = F.pad(mask, (self.num_register_tokens, 0), value = True)
+
+        # keep track of skip connections
+
         skip_connects = []
 
-        rotary_emb = self.rotary_emb(x.shape[-2])
+        # rotary embeddings
+
+        positions = seq_len
+
+        if self.has_register_tokens:
+            main_positions = torch.arange(seq_len, device = self.device, dtype = torch.long)
+            register_positions = torch.full((self.num_register_tokens,), -10000, device = self.device, dtype = torch.long)
+            positions = torch.cat((register_positions, main_positions))
+
+        rotary_emb = self.rotary_emb(positions)
+
+        # adaptive rmsnorm
 
         rmsnorm_kwargs = dict()
         if exists(adaptive_rmsnorm_cond):
             rmsnorm_kwargs = dict(cond = adaptive_rmsnorm_cond)
 
+        # going through the attention layers
+
         for skip_combiner, attn_prenorm, attn, ff_prenorm, ff in self.layers:
 
             # in the paper, they use a u-net like skip connection
@@ -376,6 +442,11 @@ class Transformer(Module):
             ff_input = ff_prenorm(x, **rmsnorm_kwargs) 
             x = ff(ff_input) + x
 
+        # remove the register tokens
+
+        if self.has_register_tokens:
+            _, x = unpack(x, ps, 'b * d')
+
         return self.final_norm(x)
 
 # encoder decoders
@@ -503,6 +574,7 @@ class DurationPredictor(Module):
         dim_head = 64,
         heads = 8,
         ff_mult = 4,
+        attn_qk_norm = True,
         conv_pos_embed_kernel_size = 31,
         conv_pos_embed_groups = None,
         attn_dropout=0,
@@ -556,7 +628,8 @@ class DurationPredictor(Module):
             heads = heads,
             ff_mult = ff_mult,
             attn_dropout=attn_dropout,
-            attn_flash = attn_flash
+            attn_flash = attn_flash,
+            attn_qk_norm = attn_qk_norm
         )
 
         self.to_pred = nn.Sequential(
@@ -781,8 +854,10 @@ class VoiceBox(Module):
         time_hidden_dim = None,
         conv_pos_embed_kernel_size = 31,
         conv_pos_embed_groups = None,
-        attn_dropout=0,
+        attn_dropout = 0,
         attn_flash = False,
+        attn_qk_norm = True,
+        num_register_tokens = 16,
         p_drop_prob = 0.3, # p_drop in paper
         frac_lengths_mask: Tuple[float, float] = (0.7, 1.),
         condition_on_text = True
@@ -837,8 +912,10 @@ class VoiceBox(Module):
             dim_head = dim_head,
             heads = heads,
             ff_mult = ff_mult,
-            attn_dropout=attn_dropout,
+            attn_dropout= attn_dropout,
             attn_flash = attn_flash,
+            attn_qk_norm = attn_qk_norm,
+            num_register_tokens = num_register_tokens,
             adaptive_rmsnorm = True,
             adaptive_rmsnorm_cond_dim_in = time_hidden_dim
         )
@@ -1031,7 +1108,8 @@ class ConditionalFlowMatcherWrapper(Module):
         self.text_to_semantic = text_to_semantic
         self.duration_predictor = duration_predictor
 
-        assert exists(text_to_semantic) ^ exists(duration_predictor), 'you should use either TextToSemantic from Spear-TTS, or DurationPredictor for the text / phoneme to audio alignment, but not both'
+        if self.condition_on_text:
+            assert exists(text_to_semantic) ^ exists(duration_predictor), 'you should use either TextToSemantic from Spear-TTS, or DurationPredictor for the text / phoneme to audio alignment, but not both'
 
         self.cond_drop_prob = cond_drop_prob
 
